#[cfg(feature = "derive")]
pub use char_enum_derive;

use std::{error::Error, fmt::Display};

#[derive(Debug, Clone, PartialEq)]
pub struct FromCharError(pub char);

impl Display for FromCharError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "invalid char: {}", self.0)
    }
}

impl Error for FromCharError {}

#[derive(Debug, Clone, PartialEq)]
pub struct FromStrError(pub String);

impl Display for FromStrError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "invalid string: {}", self.0)
    }
}

impl Error for FromStrError {}

impl From<FromCharError> for FromStrError {
    fn from(value: FromCharError) -> Self {
        Self(value.to_string())
    }
}

pub trait FromChar
where
    Self: Sized,
{
    type Err;

    fn from_char(c: char) -> Result<Self, Self::Err>;
}

pub trait ToChar {
    fn to_char(&self) -> char;
}

pub trait AsIter<T> {
    fn iter() -> impl Iterator<Item = T>;
}
