pub(crate) mod id_mapping;
pub(crate) mod mount;
pub mod permission;
pub mod stackable;

use std::{fmt::Display, slice::Iter};

pub trait AsIter<'a> {
    type Item: 'a;
    type IntoIter: Iterator<Item = &'a Self::Item>;

    fn as_iter(&'a self) -> Self::IntoIter;
}

impl<'a, T: 'a> AsIter<'a> for Vec<T> {
    type Item = T;
    type IntoIter = Iter<'a, T>;

    fn as_iter(&'a self) -> Self::IntoIter {
        self.iter()
    }
}

impl<'a, T> AsIter<'a> for &'a [T] {
    type Item = T;
    type IntoIter = Iter<'a, T>;

    fn as_iter(&'a self) -> Self::IntoIter {
        self.iter()
    }
}

pub trait FileSystem {
    fn mount(&mut self) -> std::io::Result<()>;
    fn unmount(&mut self) -> std::io::Result<()>;
}

pub trait Mountpoint {
    fn options(&self) -> impl Iterator<Item = impl Display>;
}
